/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "CountryCodes.js" as Countries

Component {

    DefaultSheet {
        id: addLocationSheet
        objectName: "AddLocationSheet"
        title: i18n.tr("Add city")
        contentsHeight: parent.height
        doneButton: false

        Component.onCompleted: {
            locationsRemoved.clear()
            // TODO final cities list
            var preDefined = [
                {"coord":{"lon":116.397232,"lat":39.907501},"service_id":1816670,"name":"Beijing","country":"CN","service":"openweathermap"},
                {"coord":{"lon":-58.377232,"lat":-34.613152},"service_id":3435910,"name":"Buenos Aires","country":"AR","service":"openweathermap"},
                {"coord":{"lon":31.24967,"lat":30.06263},"service_id":360630,"name":"Cairo","country":"EG","service":"openweathermap"},
                {"coord":{"lon":18.422,"lat":-33.9191},"service_id":3369157,"name":"Cape Town","country":"ZA","service":"openweathermap"},
                {"coord":{"lon":10,"lat":53.549999},"service_id":2911298,"name":"Hamburg","country":"DE","service":"openweathermap"},
                {"coord":{"lon":73.043289,"lat":33.721481},"service_id":1176615,"name":"Islamabad","country":"PK","service":"openweathermap"},
                {"coord":{"lon":-0.12574,"lat":51.50853},"service_id":2643743,"name":"London","country":"GB","service":"openweathermap"},
                {"coord":{"lon":-118.243683,"lat":34.052231},"service_id":5368361,"name":"Los Angeles","country":"US","service":"openweathermap"},
                {"coord":{"lon":-73.587807,"lat":45.508839},"service_id":6077243,"name":"Montreal","country":"CA","service":"openweathermap"},
                {"coord":{"lon":-99.127663,"lat":19.428471},"service_id":3530597,"name":"Mexico City","country":"MX","service":"openweathermap"},
                {"coord":{"lon":37.615001678467,"lat":55.756999969482},"service_id":524901,"name":"Moscow","country":"Russia","service":"openweathermap"},
                {"coord":{"lon":72.847939,"lat":19.01441},"service_id":1275339,"name":"Mumbai","country":"IN","service":"openweathermap"},
                {"coord":{"lon":36.816669,"lat":-1.28333},"service_id":184745,"name":"Nairobi","country":"KE","service":"openweathermap"},
                {"coord":{"lon":-74.005966,"lat":40.714272},"service_id":5128581,"name":"New York","country":"US","service":"openweathermap"},
                {"coord":{"lon":77.23114,"lat":28.61282},"service_id":1261481,"name":"New Delhi","country":"IN","service":"openweathermap"},
                {"coord":{"lon":-122.419418,"lat":37.774929},"service_id":5391959,"name":"San Francisco","country":"US","service":"openweathermap"},
                {"coord":{"lon":121.458061,"lat":31.222219},"service_id":1796236,"name":"Shanghai","country":"CN","service":"openweathermap"},
                {"coord":{"lon":103.850067,"lat":1.28967},"service_id":1880252,"name":"Singapore","country":"SG","service":"openweathermap"},
                {"coord":{"lon":151.207321,"lat":-33.867851},"service_id":2147714,"name":"Sydney","country":"AU","service":"openweathermap"},
                {"coord":{"lon":139.691711,"lat":35.689499},"service_id":1850147,"name":"Tokyo","country":"JP","service":"openweathermap"}
            ].forEach(function(loc) {
                citiesModel.append(loc)
            })
            Theme.palette.selected.backgroundText = "#656565" //color: Theme.palette.normal.baseText
        }

        WorkerScript {
            id: searchWorker
            source: Qt.resolvedUrl("./WeatherApi.js")
            onMessage: {
                searching.running = false;
                if(!messageObject.error) {
                    listView.visible = true
                    messageObject.result.locations.forEach(function(loc) {
                        citiesModel.append(loc);
                        noCityError.visible = false
                    });
                    if (!citiesModel.count) {
                        noCityError.visible = true
                        noCityError.text = i18n.tr("No location was found for ") + locationString.text
                    }
                } else {
                    console.log(messageObject.error.msg+" / "+messageObject.error.request.url)
                    httpFailedDialog.show();
                }
            }
        }

        function clear() {
            locationString.text = '';
            citiesModel.clear();
            listView.visible = true;
        }

        function doSearch() {
            if(locationString.text !== "") {
                cityList.visible = true
                citiesModel.clear();
                searching.running = true;
                searchWorker.sendMessage({
                    action: "searchByName",
                    params: {name:locationString.text, units:"metric"}
                })
            }
        }

        ListModel {
            id: citiesModel
        }

        container: Item {
            width: parent.width
            Rectangle {
                id: searchInput
                width:parent.width-units.gu(2)
                height:units.gu(5)
                anchors.horizontalCenter: parent.horizontalCenter
                color: "transparent"
                TextField {
                    id: locationString
                    objectName: "SearchField"
                    width: parent.width
                    height:parent.height
                    placeholderText: i18n.tr("Enter a city name")
                    hasClearButton: true
                    onAccepted: {
                        doSearch();
                        search_timer.stop()
                        locationString.focus = false
                    }
                    primaryItem: Image {
                        height: parent.height*0.5
                        width: parent.height*0.5
                        anchors.verticalCenter: parent.verticalCenter
                        anchors.verticalCenterOffset: -units.gu(0.2)
                        source: Qt.resolvedUrl("../resources/images/search.svg")
                    }

                    Timer {
                        id: search_timer

                        interval: 1100
                        repeat: false
                        onTriggered: {
                            if (locationString.text != "") {
                                doSearch();
                            }
                        }
                    }

                    onTextChanged: search_timer.restart()
                }
                ActivityIndicator {
                    id: searching
                    objectName: "SearchingSpinner"
                    anchors {
                        right: locationString.right
                        rightMargin: units.gu(2)
                        verticalCenter: locationString.verticalCenter
                    }
                }
            }

            Rectangle {
                id: cityList;
                anchors.top: searchInput.bottom
                width: parent.width
                height: addLocationSheet.height-searchInput.height-units.gu(9.5)
                color: "transparent"
                visible: true
                Label {
                    id: noCityError
                    objectName: "noCityError"
                    visible: false
                    anchors.centerIn: parent
                    fontSize: "small"
                }
                Component {
                    id: sectionHeading
                    ListItem.Header {
                        Label {
                            text: section
                            anchors {
                                verticalCenter: parent.verticalCenter
                                left: parent.left
                                leftMargin: units.gu(1)
                            }
                            fontSize: "medium"
                        }
                    }
                }
                ListView {
                    id: listView;
                    objectName: "SearchResultList"
                    clip: true;
                    anchors.fill: parent;
                    model:  citiesModel;
                    section.property: "name"
                    section.criteria: ViewSection.FirstCharacter
                    section.delegate: sectionHeading;
                    section.labelPositioning: ViewSection.InlineLabels
                    delegate: ListItem.Standard {
                        Label {
                            id: searchResultName
                            objectName: "searchResult" + index
                            text: name
                            anchors {
                                top: parent.top
                                topMargin: units.gu(0.5)
                                left: parent.left
                                leftMargin: units.gu(3)
                            }
                            fontSize: "large"
                        }
                        Label {
                            text: (Countries.codes[country]) ? i18n.tr(Countries.codes[country]): country
                            anchors {
                                top: searchResultName.bottom
                                bottom: parent.bottom
                                left: parent.left
                                leftMargin: units.gu(3)
                            }
                            fontSize: "small"
                        }
                        onClicked: {
                            var location = citiesModel.get(index)
                            locationManagerSheet.addLocation(location)
                            PopupUtils.close(addLocationSheet)
                            clear()
                        }
                    }
                    Scrollbar {
                        flickableItem: listView;
                        align: Qt.AlignTrailing;
                    }
                }
            }
        }
    }
}
