/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components 0.1

Tab {
    id: locationTab
    objectName: "LocationTab"

    anchors.fill: parent

    property int locationIndex: 0
    property var locationData: {}

    function formatTimestamp(timestamp, format) {
        var date = new Date(timestamp*1000);
        return Qt.formatDate(date, format)
    }

    function formatTimestampToTime(timestamp, format) {
        var date = new Date(timestamp*1000);
        return Qt.formatTime(date, format)
    }

    onVisibleChanged: {
        var currentIndex = dailyForecastList.currentIndex;
        if(visible == true && currentIndex > -1) {
            var data = dayForecastModel.get(currentIndex)
            adjustBackground(Math.round(data.temp))
        }
    }

    Component.onCompleted: {
        var locData = mainView.locationsList[locationIndex],
            units = (mainView.settings.units === 'imperial') ? 'imperial' : 'metric',
            wind_units = (mainView.settings.wind_units === 'mph') ? 'imperial' : 'metric',
            dailyForecasts = locData.data,
            dailyLength = dailyForecasts.length;

        // set location data as property
        locationData = locData;

        // set daily forecasts
        for(var x=0;x<dailyLength;x++) {
            var direction = "?";
            var degrees = dailyForecasts[x].windDeg;
            if(degrees >=0 && degrees <= 30){
                direction = "N";
            } else if(degrees >30 && degrees <= 60){
                direction = "NE";
            } else if(degrees >60 && degrees <= 120){
                direction = "E";
            } else if(degrees >120 && degrees <= 150){
                direction = "SE";
            } else if(degrees >150 && degrees <= 210){
                direction = "S";
            } else if(degrees >210 && degrees <= 240){
                direction = "SW";
            } else if(degrees >240 && degrees <= 300){
                direction = "W";
            } else if(degrees >300 && degrees <= 330){
                direction = "NW";
            }else if(degrees >330 && degrees <= 360){
                direction = "N";
            }
            // set last updated label
            if(x===0 && dailyForecasts[x].current) {
                lastUpdatedComponent.updateDate = dailyForecasts[x].current.timestamp;
            }
            //
            dayForecastModel.append({
                dateRel: "",//Tomorrow",
                date: formatTimestamp(dailyForecasts[x].timestamp, 'dddd, dd MMMM'),
                temp: (dailyForecasts[x]["current"] === undefined) ? dailyForecasts[x][units].tempMax :
                                            dailyForecasts[x]["current"][units].temp,
                tempMin: dailyForecasts[x][units].tempMin,
                tempMax: (dailyForecasts[x]["current"] !== undefined) ? dailyForecasts[x][units].tempMax :
                                            null,
                cond: dailyForecasts[x].condition.id,
                condIcon: dailyForecasts[x].icon,
                wind_speed: dailyForecasts[x][wind_units].windSpeed,
                wind_dir: direction,
                humid: dailyForecasts[x].humidity,
                precip: +(dailyForecasts[x][units].rain || dailyForecasts[x][units].snow || 0).toFixed(2),
                press: dailyForecasts[x].pressure,
                hours: dailyForecasts[x]["hourly"]
            });
        }
    }

    // Menu for options
    page: Page {
        id: locationPage
        isLeaf: true
        anchors.fill: parent
        anchors.topMargin: units.gu(9.5)

        ListModel {
            id: dayForecastModel
        }

        ListView {
            id: dailyForecastList
            width: parent.width
            height:parent.height
            model:dayForecastModel

            onCurrentIndexChanged: {
                if(visible) {
                    var data = dayForecastModel.get(currentIndex)
                    adjustBackground(Math.round(data.temp))
                }
            }

            interactive: false
            snapMode: ListView.SnapOneItem
            highlightRangeMode: ListView.StrictlyEnforceRange
            verticalLayoutDirection: ListView.TopToBottom
            flickableDirection: Flickable.VerticalFlick
            clip: true

            delegate: Item {
                id: dailyForecastItem
                width:parent.width
                height: locationPage.height

                Rectangle {
                    id: listRectangle
                    width: parent.width
                    height: dailyForecastList.height
                    anchors.verticalCenter: parent.verticalCenter
                    color: "transparent"

                    DateComponent {
                        id: dateComponent
                        dateRelative: dateRel
                        dateString: date
                        borderTopVisible: false
                    }
                    CurrentWeather {
                        currentTemp: temp
                        minTemp: tempMin
                        maxTemp: tempMax
                        icon: condIcon
                        condition: cond
                        windSpeed: wind_speed
                        windDir: wind_dir
                        humidity: humid
                        precipitation: precip
                        pressure: press
                        hourly: hours
                        anchors.top: dateComponent.bottom
                        showMaxTemp: (index === 0)
                    }
                }
            }
            LastUpdatedComponent{
                id: lastUpdatedComponent
            }
        }


        tools: ToolbarItems {
            id: pageTools
            ToolbarButton {
                objectName: "RefreshButton"
                action:Action {
                    id: refreshAction
                    objectName: "RefreshAction"
                    iconSource: Qt.resolvedUrl("../resources/images/refresh_icon.png")
                    text: i18n.tr("Refresh")

                    onTriggered: {
                        mainView.refreshData(false, true);
                    }
                }
            }
            ToolbarButton {
                objectName: "EditButton"
                action:Action {
                    id: editLocationAction
                    iconSource: Qt.resolvedUrl("../resources/images/add_icon.png")
                    text: i18n.tr("Edit")

                    onTriggered: {
                        mainView.showLocationManager()
                    }
                }
            }
            ToolbarButton {
                objectName: "SettingsButton"
                action:Action {
                    id: configAction
                    iconSource: Qt.resolvedUrl("../resources/images/settings.png")
                    text: i18n.tr("Settings")

                    onTriggered: {
                        mainView.showSettings();
                    }
                }
            }
        }
    }
}
